@@ -2,18 +2,26 @@
 
 import asyncio
 from typing import (
-    Any,
-    Mapping,
-    Optional
+    Optional,
+    cast
 )
 
 import h11
 
-from .http_protocol import HttpProtocol
-from .utils import get_target
 from .asyncio_events import MessageEvent
+from .http_protocol import HttpProtocol
+from .types import (
+    HttpRequest,
+    HttpRequestBody,
+    HttpResponse,
+    HttpResponseConnection,
+    HttpResponseBody,
+    HttpDisconnect,
+    HttpRequests,
+    HttpResponses
+)
 
-MappingMessageEvent = MessageEvent[Mapping[str, Any]]
+MappingMessageEvent = MessageEvent[HttpResponses]
 
 
 class H11Protocol(HttpProtocol):
@@ -45,20 +53,20 @@ class H11Protocol(HttpProtocol):
             self._h11_state.start_next_cycle()
         self._is_message_ended = False
 
-    async def send(self, message: Mapping[str, Any]) -> None:
+    async def send(self, message: HttpRequests) -> None:
 
         request_type: str = message['type']
 
         if request_type == 'http.request':
-            await self._send_request(message)
+            await self._send_request(cast(HttpRequest, message))
         elif request_type == 'http.request.body':
-            await self._send_request_body(message)
+            await self._send_request_body(cast(HttpRequestBody, message))
         elif request_type == 'http.disconnect':
             await self._disconnect()
         else:
             raise Exception(f'unknown request type: {request_type}')
 
-    async def receive(self) -> Mapping[str, Any]:
+    async def receive(self) -> HttpResponses:
 
         message = await self._connection_event.wait_with_message()
         if message is not None:
@@ -71,32 +79,33 @@ class H11Protocol(HttpProtocol):
         message = await self._receive_body_event()
         return message
 
-    async def _send_request(self, message: Mapping[str, Any]) -> None:
+    async def _send_request(self, message: HttpRequest) -> None:
 
         self._connect()
         self._is_initialised = True
 
         request = h11.Request(
             method=message['method'],
-            target=get_target(message['url']),
+            target=message['path'],
             headers=message.get('headers', [])
         )
 
         buf = self._h11_state.send(request)
         self.writer.write(buf)
         await self.writer.drain()
-        self._connection_event.set_with_message({
+        http_response_connection: HttpResponseConnection = {
             'type': 'http.response.connection',
             'http_version': 'h11',
             'stream_id': None
-        })
+        }
+        self._connection_event.set_with_message(http_response_connection)
 
-        body = message.get('body', b'')
-        more_body = message.get('more_body', False)
+        body = message['body']
+        more_body = message['more_body']
         await self._send_request_data(body, more_body)
         asyncio.create_task(self._receive_response())
 
-    async def _send_request_body(self, message: Mapping[str, Any]) -> None:
+    async def _send_request_body(self, message: HttpRequestBody) -> None:
         await self._send_request_data(
             message.get('body', b''),
             message.get('more_body', False)
@@ -104,11 +113,12 @@ class H11Protocol(HttpProtocol):
 
     async def _send_request_data(
             self,
-            body: bytes,
+            body: Optional[bytes],
             more_body: Optional[bool]
     ) -> None:
-        buf = self._h11_state.send(h11.Data(data=body))
-        self.writer.write(buf)
+        if body is not None:
+            buf = self._h11_state.send(h11.Data(data=body))
+            self.writer.write(buf)
 
         if not more_body:
             buf = self._h11_state.send(h11.EndOfMessage())
@@ -136,7 +146,7 @@ class H11Protocol(HttpProtocol):
             elif name == b'transfer-encoding' and value == b'chunked':
                 more_body = True
 
-        self._response_event.set_with_message({
+        http_response: HttpResponse = {
             'type': 'http.response',
             'acgi': {
                 'version': "1.0"
@@ -146,7 +156,8 @@ class H11Protocol(HttpProtocol):
             'headers': event.headers,
             'more_body': more_body,
             'stream_id': None
-        })
+        }
+        self._response_event.set_with_message(http_response)
 
     async def _disconnect(self) -> None:
         if not self._is_message_ended and self._h11_state.our_state != h11.DONE:
@@ -163,30 +174,33 @@ class H11Protocol(HttpProtocol):
         self.writer.close()
         await self.writer.wait_closed()
 
-    async def _receive_body_event(self) -> Mapping[str, Any]:
+    async def _receive_body_event(self) -> HttpResponses:
         while True:
             event = self._h11_state.next_event()
             if event is h11.NEED_DATA:
                 self._h11_state.receive_data(await self.reader.read(self._bufsiz))
             elif isinstance(event, h11.Data):
-                return {
+                http_response_body: HttpResponseBody = {
                     'type': 'http.response.body',
                     'body': event.data,
                     'more_body': True,
                     'stream_id': None
                 }
+                return http_response_body
             elif isinstance(event, h11.EndOfMessage):
                 self._is_message_ended = True
-                return {
+                http_response_body = {
                     'type': 'http.response.body',
                     'body': b'',
                     'more_body': False,
                     'stream_id': None
                 }
+                return http_response_body
             elif isinstance(event, h11.ConnectionClosed):
-                return {
-                    'type': 'http.stream.disconnect',
+                http_disconnect: HttpDisconnect = {
+                    'type': 'http.disconnect',
                     'stream_id': None
                 }
+                return http_disconnect
             else:
                 raise ValueError('Unknown event')
