@@ -3,159 +3,36 @@
 from asyncio import AbstractEventLoop
 import json
 import ssl
-from typing import Any, Callable, Iterable, Mapping, Optional, Type, Union
-from urllib.error import HTTPError
-from urllib.parse import urlparse
+from typing import Any, Callable, Iterable, List, Optional, Tuple, Union
 
-from baretypes import Headers
-import bareutils.header as header
-from bareutils import bytes_writer
-from bareutils.compression import Decompressor
+from bareutils import bytes_writer, text_writer, header
 
 from .client import HttpClient
-from .constants import USER_AGENT, DEFAULT_PROTOCOLS
+from .constants import DEFAULT_PROTOCOLS
 from .ssl_contexts import DEFAULT_CIPHERS, DEFAULT_OPTIONS
-
-
-async def request(
-        url: str,
-        method: str,
-        *,
-        headers: Headers = None,
-        content: Optional[bytes] = None,
-        loop: Optional[AbstractEventLoop] = None,
-        cafile: Optional[str] = None,
-        capath: Optional[str] = None,
-        cadata: Optional[str] = None,
-        ssl_context: Optional[ssl.SSLContext] = None,
-        decompressors: Optional[Mapping[bytes, Type[Decompressor]]] = None,
-        protocols: Iterable[str] = DEFAULT_PROTOCOLS,
-        ciphers: Iterable[str] = DEFAULT_CIPHERS,
-        options: Iterable[int] = DEFAULT_OPTIONS,
-        chunk_size: int = -1,
-        connect_timeout: Optional[Union[int, float]] = None
-) -> bytes:
-    """Gets bytes from a url.
-
-    ```python
-    buf = await request(
-        'https://jsonplaceholder.typicode.com/todos/1',
-        'GET',
-        ssl=ssl.SSLContext()
-    )
-    ```
-
-    Args:
-        url (str): The url to get.
-        method (str): The HTTP method (eg. 'GET', 'POST', etc).
-        headers (Headers, optional): Any extra headers required. Defaults to
-            None.
-        content (Optional[bytes], optional): The content to send.. Defaults to
-            None.
-        loop (Optional[AbstractEventLoop], optional): The optional asyncio event
-            loop.. Defaults to None.
-        cafile (Optional[str], optional): The path to a file of concatenated CA
-            certificates in PEM format. Defaults to None.
-        capath (Optional[str], optional): The path to a directory containing
-            several CA certificates in PEM format. Defaults to None.
-        cadata (Optional[str], optional): Either an ASCII string of one or more
-            PEM-encoded certificates or a bytes-like object of DER-encoded
-            certificates. Defaults to None.
-        ssl_context (Optional[SSLContext], optional): An ssl context to be
-            used instead of generating one from the certificates.
-        decompressors (Optional[Mapping[bytes, Type[Decompressor]]], optional):
-            The decompressors. Defaults to None.
-        protocols (Iterable[str], optional): The supported protocols. Defaults
-            to DEFAULT_PROTOCOLS.
-        ciphers (Iterable[str], optional): The supported ciphers. Defaults
-            to DEFAULT_CIPHERS.
-        options (Iterable[int], optional): The ssl.SSLContext.options. Defaults
-            to DEFAULT_OPTIONS.
-        chunk_size (int, optional): The size of each chunk to send or -1 to send
-            as a single chunk. Defaults to -1.
-        connect_timeout (Optional[Union[int, float]], optional): The number
-            of seconds to wait for the connection. Defaults to None.
-
-    Raises:
-        HTTPError: Is the status code is not ok.
-        asyncio.TimeoutError: If the connect times out.
-
-    Returns:
-        bytes: The bytes received.
-    """
-
-    headers = [] if headers is None else list(headers)
-
-    content_length = str(len(content)).encode('ascii') if content else b'0'
-
-    hostname = urlparse(url).hostname
-    if hostname is None:
-        raise RuntimeError('unspecified hostname')
-
-    base_headers = [
-        (b'host', hostname.encode('ascii')),
-        (b'content-length', content_length),
-        (b'user-agent', USER_AGENT),
-        (b'connection', b'close')
-    ]
-
-    for name, value in base_headers:
-        if not header.find(name, headers):
-            headers.append((name, value))
-
-    data = bytes_writer(content, chunk_size) if content else None
-
-    async with HttpClient(
-            url,
-            method=method,
-            headers=headers,
-            content=data,
-            loop=loop,
-            cafile=cafile,
-            capath=capath,
-            cadata=cadata,
-            ssl_context=ssl_context,
-            decompressors=decompressors,
-            protocols=protocols,
-            ciphers=ciphers,
-            options=options,
-            connect_timeout=connect_timeout
-    ) as response:
-        buf = b''
-        if response['more_body']:
-            async for part in response['body']:
-                buf += part
-        if response['status_code'] < 200 or response['status_code'] >= 400:
-            raise HTTPError(
-                url,
-                response['status_code'],
-                buf,
-                response['headers'],
-                None
-            )
-        return buf
+from .middleware import HttpClientMiddlewareCallback
 
 
 async def get(
         url: str,
         *,
-        headers: Headers = None,
+        headers: List[Tuple[bytes, bytes]] = None,
         loop: Optional[AbstractEventLoop] = None,
         cafile: Optional[str] = None,
         capath: Optional[str] = None,
         cadata: Optional[str] = None,
         ssl_context: Optional[ssl.SSLContext] = None,
-        decompressors: Optional[Mapping[bytes, Type[Decompressor]]] = None,
         protocols: Iterable[str] = DEFAULT_PROTOCOLS,
         ciphers: Iterable[str] = DEFAULT_CIPHERS,
         options: Iterable[int] = DEFAULT_OPTIONS,
-        connect_timeout: Optional[Union[int, float]] = None
-) -> bytes:
+        connect_timeout: Optional[Union[int, float]] = None,
+        middleware: Optional[List[HttpClientMiddlewareCallback]] = None
+) -> Optional[bytes]:
     """Issues a GET request
 
     Args:
         url (str): The url
-        headers (Headers, optional): Any extra headers required. Defaults to
+        headers (List[Tuple[bytes, bytes]], optional): Any extra headers required. Defaults to
             None.
         loop (Optional[AbstractEventLoop], optional): The optional asyncio event
             loop.. Defaults to None.
@@ -168,8 +45,6 @@ async def get(
             certificates. Defaults to None.
         ssl_context (Optional[SSLContext], optional): An ssl context to be
             used instead of generating one from the certificates.
-        decompressors (Optional[Mapping[bytes, Type[Decompressor]]], optional):
-            The decompressors. Defaults to None.
         protocols (Iterable[str], optional): The supported protocols. Defaults
             to DEFAULT_PROTOCOLS.
         ciphers (Iterable[str], optional): The supported ciphers. Defaults
@@ -178,47 +53,51 @@ async def get(
             to DEFAULT_OPTIONS.
         connect_timeout (Optional[Union[int, float]], optional): The number
             of seconds to wait for the connection. Defaults to None.
+        middleware (Optional[List[HttpClientMiddlewareCallback]], optional):
+            Optional middleware. Defaults to None.
 
     Raises:
         HTTPError: Is the status code is not ok.
         asyncio.TimeoutError: If the connect times out.
 
     Returns:
-        bytes: [description]
+        Optional[bytes]: [description]
     """
-    return await request(
-        url,
-        'GET',
-        headers=headers,
-        loop=loop,
-        cafile=cafile,
-        capath=capath,
-        cadata=cadata,
-        ssl_context=ssl_context,
-        decompressors=decompressors,
-        protocols=protocols,
-        ciphers=ciphers,
-        options=options,
-        connect_timeout=connect_timeout
-    )
+    async with HttpClient(
+            url,
+            method='GET',
+            headers=headers,
+            loop=loop,
+            cafile=cafile,
+            capath=capath,
+            cadata=cadata,
+            ssl_context=ssl_context,
+            protocols=protocols,
+            ciphers=ciphers,
+            options=options,
+            connect_timeout=connect_timeout,
+            middleware=middleware
+    ) as response:
+        await response.raise_for_status()
+        return await response.raw()
 
 
 async def get_text(
         url: str,
         *,
-        headers: Headers = None,
+        headers: List[Tuple[bytes, bytes]] = None,
         loop: Optional[AbstractEventLoop] = None,
         encoding: str = 'utf-8',
         cafile: Optional[str] = None,
         capath: Optional[str] = None,
         cadata: Optional[str] = None,
         ssl_context: Optional[ssl.SSLContext] = None,
-        decompressors: Optional[Mapping[bytes, Type[Decompressor]]] = None,
         protocols: Iterable[str] = DEFAULT_PROTOCOLS,
         ciphers: Iterable[str] = DEFAULT_CIPHERS,
         options: Iterable[int] = DEFAULT_OPTIONS,
-        connect_timeout: Optional[Union[int, float]] = None
-) -> str:
+        connect_timeout: Optional[Union[int, float]] = None,
+        middleware: Optional[List[HttpClientMiddlewareCallback]] = None
+) -> Optional[str]:
     """Issues a GET request returning a string
 
     The following gets some text:
@@ -236,7 +115,7 @@ async def get_text(
 
     Args:
         url (str): The url
-        headers (Headers, optional): Any extra headers required. Defaults to
+        headers (List[Tuple[bytes, bytes]], optional): Any extra headers required. Defaults to
             None.
         loop (Optional[AbstractEventLoop], optional): The optional asyncio event
             loop.. Defaults to None.
@@ -249,8 +128,6 @@ async def get_text(
             certificates. Defaults to None.
         ssl_context (Optional[SSLContext], optional): An ssl context to be
             used instead of generating one from the certificates.
-        decompressors (Optional[Mapping[bytes, Type[Decompressor]]], optional):
-            The decompressors. Defaults to None.
         protocols (Iterable[str], optional): The supported protocols. Defaults
             to DEFAULT_PROTOCOLS.
         ciphers (Iterable[str], optional): The supported ciphers. Defaults
@@ -259,53 +136,57 @@ async def get_text(
             to DEFAULT_OPTIONS.
         connect_timeout (Optional[Union[int, float]], optional): The number
             of seconds to wait for the connection. Defaults to None.
+        middleware (Optional[List[HttpClientMiddlewareCallback]], optional):
+            Optional middleware. Defaults to None.
 
     Raises:
         HTTPError: Is the status code is not ok.
         asyncio.TimeoutError: If the connect times out.
 
     Returns:
-        str: [description]
+        Optional[str]: [description]
     """
 
     headers = [] if headers is None else list(headers)
 
-    if not header.find(b'accept', headers):
-        headers.append((b'accept', b'text/plain'))
+    if not header.find(header.ACCEPT, headers):
+        headers.append((header.ACCEPT, b'text/plain'))
 
-    buf = await get(
-        url,
-        headers=headers,
-        loop=loop,
-        cafile=cafile,
-        capath=capath,
-        cadata=cadata,
-        ssl_context=ssl_context,
-        decompressors=decompressors,
-        protocols=protocols,
-        ciphers=ciphers,
-        options=options,
-        connect_timeout=connect_timeout
-    )
-    return buf.decode(encoding)
+    async with HttpClient(
+            url,
+            method='GET',
+            headers=headers,
+            loop=loop,
+            cafile=cafile,
+            capath=capath,
+            cadata=cadata,
+            ssl_context=ssl_context,
+            protocols=protocols,
+            ciphers=ciphers,
+            options=options,
+            connect_timeout=connect_timeout,
+            middleware=middleware
+    ) as response:
+        await response.raise_for_status()
+        return await response.text(encoding)
 
 
 async def get_json(
         url: str,
         *,
-        headers: Headers = None,
-        loads: Callable[[str], Any] = json.loads,
+        headers: List[Tuple[bytes, bytes]] = None,
+        loads: Callable[[bytes], Any] = json.loads,
         loop: Optional[AbstractEventLoop] = None,
         cafile: Optional[str] = None,
         capath: Optional[str] = None,
         cadata: Optional[str] = None,
         ssl_context: Optional[ssl.SSLContext] = None,
-        decompressors: Optional[Mapping[bytes, Type[Decompressor]]] = None,
         protocols: Iterable[str] = DEFAULT_PROTOCOLS,
         ciphers: Iterable[str] = DEFAULT_CIPHERS,
         options: Iterable[int] = DEFAULT_OPTIONS,
-        connect_timeout: Optional[Union[int, float]] = None
-) -> Any:
+        connect_timeout: Optional[Union[int, float]] = None,
+        middleware: Optional[List[HttpClientMiddlewareCallback]] = None
+) -> Optional[Any]:
     """Issues a GET request returning a JSON object
 
     The following gets some json:
@@ -323,9 +204,9 @@ async def get_json(
 
     Args:
         url (str): The url
-        headers (Headers, optional): Any extra headers required. Defaults to
+        headers (List[Tuple[bytes, bytes]], optional): Any extra headers required. Defaults to
             None.
-        loads (Callable[[str], Any], optional): The function to loads the
+        loads (Callable[[bytes], Any], optional): The function to loads the
             JSON object from the string. Defaults to json.loads.
         loop (Optional[AbstractEventLoop], optional): The optional asyncio event
             loop.. Defaults to None.
@@ -338,8 +219,6 @@ async def get_json(
             certificates. Defaults to None.
         ssl_context (Optional[SSLContext], optional): An ssl context to be
             used instead of generating one from the certificates.
-        decompressors (Optional[Mapping[bytes, Type[Decompressor]]], optional):
-            The decompressors. Defaults to None.
         protocols (Iterable[str], optional): The supported protocols. Defaults
             to DEFAULT_PROTOCOLS.
         ciphers (Iterable[str], optional): The supported ciphers. Defaults
@@ -348,60 +227,63 @@ async def get_json(
             to DEFAULT_OPTIONS.
         connect_timeout (Optional[Union[int, float]], optional): The number
             of seconds to wait for the connection. Defaults to None.
+        middleware (Optional[List[HttpClientMiddlewareCallback]], optional):
+            Optional middleware. Defaults to None.
 
     Raises:
         HTTPError: Is the status code is not ok.
         asyncio.TimeoutError: If the connect times out.
 
     Returns:
-        Any: The decoded JSON object
+        Optional[Any]: The decoded JSON object
     """
     headers = [] if headers is None else list(headers)
 
-    if not header.find(b'accept', headers):
-        headers.append((b'accept', b'application/json'))
+    if not header.find(header.ACCEPT, headers):
+        headers.append((header.ACCEPT, b'application/json'))
 
-    text = await get_text(
-        url,
-        headers=headers,
-        encoding='utf-8',
-        loop=loop,
-        cafile=cafile,
-        capath=capath,
-        cadata=cadata,
-        ssl_context=ssl_context,
-        decompressors=decompressors,
-        protocols=protocols,
-        ciphers=ciphers,
-        options=options,
-        connect_timeout=connect_timeout
-    )
-    return loads(text)
+    async with HttpClient(
+            url,
+            method='GET',
+            headers=headers,
+            loop=loop,
+            cafile=cafile,
+            capath=capath,
+            cadata=cadata,
+            ssl_context=ssl_context,
+            protocols=protocols,
+            ciphers=ciphers,
+            options=options,
+            connect_timeout=connect_timeout,
+            middleware=middleware
+    ) as response:
+        await response.raise_for_status()
+        return await response.json(loads)
 
 
 async def post(
         url: str,
         content: bytes,
         *,
-        headers: Headers = None,
+        headers: List[Tuple[bytes, bytes]] = None,
         loop: Optional[AbstractEventLoop] = None,
         cafile: Optional[str] = None,
         capath: Optional[str] = None,
         cadata: Optional[str] = None,
         ssl_context: Optional[ssl.SSLContext] = None,
-        decompressors: Optional[Mapping[bytes, Type[Decompressor]]] = None,
         protocols: Iterable[str] = DEFAULT_PROTOCOLS,
         ciphers: Iterable[str] = DEFAULT_CIPHERS,
         options: Iterable[int] = DEFAULT_OPTIONS,
         chunk_size: int = -1,
-        connect_timeout: Optional[Union[int, float]] = None
-) -> bytes:
+        connect_timeout: Optional[Union[int, float]] = None,
+        middleware: Optional[List[HttpClientMiddlewareCallback]] = None
+) -> Optional[bytes]:
     """Issues a POST request
 
     Args:
         url (str): The url
         content (bytes): The body content
-        headers (Headers, optional): Any extra headers required. Defaults to
+        headers (List[Tuple[bytes, bytes]], optional): Any extra headers required. Defaults to
             None.
         loop (Optional[AbstractEventLoop], optional): The optional asyncio event
             loop.. Defaults to None.
@@ -414,8 +296,6 @@ async def post(
             certificates. Defaults to None.
         ssl_context (Optional[SSLContext], optional): An ssl context to be
             used instead of generating one from the certificates.
-        decompressors (Optional[Mapping[bytes, Type[Decompressor]]], optional):
-            The decompressors. Defaults to None.
         protocols (Iterable[str], optional): The supported protocols. Defaults
             to DEFAULT_PROTOCOLS.
         ciphers (Iterable[str], optional): The supported ciphers. Defaults
@@ -426,31 +306,36 @@ async def post(
             as a single chunk.. Defaults to -1.
         connect_timeout (Optional[Union[int, float]], optional): The number
             of seconds to wait for the connection. Defaults to None.
+        middleware (Optional[List[HttpClientMiddlewareCallback]], optional):
+            Optional middleware. Defaults to None.
 
     Raises:
         HTTPError: Is the status code is not ok.
         asyncio.TimeoutError: If the connect times out.
 
     Returns:
-        bytes: The response body
+        Optional[bytes]: The response body
     """
-    return await request(
-        url,
-        method='POST',
-        content=content,
-        headers=headers,
-        loop=loop,
-        cafile=cafile,
-        capath=capath,
-        cadata=cadata,
-        ssl_context=ssl_context,
-        decompressors=decompressors,
-        protocols=protocols,
-        ciphers=ciphers,
-        options=options,
-        chunk_size=chunk_size,
-        connect_timeout=connect_timeout
-    )
+    data = bytes_writer(content, chunk_size) if content else None
+
+    async with HttpClient(
+            url,
+            method='POST',
+            headers=headers,
+            body=data,
+            loop=loop,
+            cafile=cafile,
+            capath=capath,
+            cadata=cadata,
+            ssl_context=ssl_context,
+            protocols=protocols,
+            ciphers=ciphers,
+            options=options,
+            connect_timeout=connect_timeout,
+            middleware=middleware
+    ) as response:
+        await response.raise_for_status()
+        return await response.raw()
 
 
 async def post_text(
@@ -458,25 +343,25 @@ async def post_text(
         text: str,
         *,
         encoding='utf-8',
-        headers: Headers = None,
+        headers: List[Tuple[bytes, bytes]] = None,
         loop: Optional[AbstractEventLoop] = None,
         cafile: Optional[str] = None,
         capath: Optional[str] = None,
         cadata: Optional[str] = None,
         ssl_context: Optional[ssl.SSLContext] = None,
-        decompressors: Optional[Mapping[bytes, Type[Decompressor]]] = None,
         protocols: Iterable[str] = DEFAULT_PROTOCOLS,
         ciphers: Iterable[str] = DEFAULT_CIPHERS,
         options: Iterable[int] = DEFAULT_OPTIONS,
         chunk_size: int = -1,
-        connect_timeout: Optional[Union[int, float]] = None
-) -> str:
+        connect_timeout: Optional[Union[int, float]] = None,
+        middleware: Optional[List[HttpClientMiddlewareCallback]] = None
+) -> Optional[str]:
     """Issues a POST request with a str body
 
     Args:
         url (str): The url
         content (bytes): The body content
-        headers (Headers, optional): Any extra headers required. Defaults to
+        headers (List[Tuple[bytes, bytes]], optional): Any extra headers required. Defaults to
             None.
         loop (Optional[AbstractEventLoop], optional): The optional asyncio event
             loop.. Defaults to None.
@@ -489,8 +374,6 @@ async def post_text(
             certificates. Defaults to None.
         ssl_context (Optional[SSLContext], optional): An ssl context to be
             used instead of generating one from the certificates.
-        decompressors (Optional[Mapping[bytes, Type[Decompressor]]], optional):
-            The decompressors. Defaults to None.
         protocols (Iterable[str], optional): The supported protocols. Defaults
             to DEFAULT_PROTOCOLS.
         ciphers (Iterable[str], optional): The supported ciphers. Defaults
@@ -501,6 +384,8 @@ async def post_text(
             as a single chunk.. Defaults to -1.
         connect_timeout (Optional[Union[int, float]], optional): The number
             of seconds to wait for the connection. Defaults to None.
+        middleware (Optional[List[HttpClientMiddlewareCallback]], optional):
+            Optional middleware. Defaults to None.
 
     Raises:
         HTTPError: Is the status code is not ok.
@@ -512,50 +397,53 @@ async def post_text(
 
     headers = [] if headers is None else list(headers)
 
-    if not header.find(b'accept', headers):
-        headers.append((b'accept', b'text/plain'))
+    if not header.find(header.ACCEPT, headers):
+        headers.append((header.ACCEPT, b'text/plain'))
 
     content = text.encode(encoding=encoding)
-    if not header.find(b'content-type', headers):
-        headers.append((b'content-type', b'text/plain'))
+    if not header.find(header.CONTENT_TYPE, headers):
+        headers.append((header.CONTENT_TYPE, b'text/plain'))
 
-    response = await post(
-        url,
-        content=content,
-        headers=headers,
-        loop=loop,
-        cafile=cafile,
-        capath=capath,
-        cadata=cadata,
-        ssl_context=ssl_context,
-        decompressors=decompressors,
-        protocols=protocols,
-        ciphers=ciphers,
-        options=options,
-        chunk_size=chunk_size,
-        connect_timeout=connect_timeout
-    )
-    return response.decode(encoding=encoding)
+    data = bytes_writer(content, chunk_size) if content else None
+
+    async with HttpClient(
+            url,
+            method='POST',
+            headers=headers,
+            body=data,
+            loop=loop,
+            cafile=cafile,
+            capath=capath,
+            cadata=cadata,
+            ssl_context=ssl_context,
+            protocols=protocols,
+            ciphers=ciphers,
+            options=options,
+            connect_timeout=connect_timeout,
+            middleware=middleware
+    ) as response:
+        await response.raise_for_status()
+        return await response.text()
 
 
 async def post_json(
         url: str,
         obj: Any,
         *,
-        loads: Callable[[str], Any] = json.loads,
+        loads: Callable[[bytes], Any] = json.loads,
         dumps: Callable[[Any], str] = json.dumps,
-        headers: Headers = None,
+        headers: List[Tuple[bytes, bytes]] = None,
         loop: Optional[AbstractEventLoop] = None,
         cafile: Optional[str] = None,
         capath: Optional[str] = None,
         cadata: Optional[str] = None,
         ssl_context: Optional[ssl.SSLContext] = None,
-        decompressors: Optional[Mapping[bytes, Type[Decompressor]]] = None,
         protocols: Iterable[str] = DEFAULT_PROTOCOLS,
         ciphers: Iterable[str] = DEFAULT_CIPHERS,
         options: Iterable[int] = DEFAULT_OPTIONS,
         chunk_size: int = -1,
-        connect_timeout: Optional[Union[int, float]] = None
+        connect_timeout: Optional[Union[int, float]] = None,
+        middleware: Optional[List[HttpClientMiddlewareCallback]] = None
 ) -> Optional[Any]:
     """Issues a POST request with a JSON payload
 
@@ -577,11 +465,11 @@ async def post_json(
     Args:
         url (str): The url
         obj (Any): The JSON payload
-        loads (Callable[[str], Any], optional): The function used to decode
+        loads (Callable[[bytes], Any], optional): The function used to decode
             the response. Defaults to json.loads.
         dumps (Callable[[Any], str], optional): The function used to encode
             the request. Defaults to json.dumps.
-        headers (Headers, optional): Any extra headers required. Defaults to
+        headers (List[Tuple[bytes, bytes]], optional): Any extra headers required. Defaults to
             None.
         loop (Optional[AbstractEventLoop], optional): The optional asyncio event
             loop.. Defaults to None.
@@ -594,8 +482,6 @@ async def post_json(
             certificates. Defaults to None.
         ssl_context (Optional[SSLContext], optional): An ssl context to be
             used instead of generating one from the certificates.
-        decompressors (Optional[Mapping[bytes, Type[Decompressor]]], optional):
-            The decompressors. Defaults to None.
         protocols (Iterable[str], optional): The supported protocols. Defaults
             to DEFAULT_PROTOCOLS.
         ciphers (Iterable[str], optional): The supported ciphers. Defaults
@@ -606,6 +492,8 @@ async def post_json(
             as a single chunk.. Defaults to -1.
         connect_timeout (Optional[Union[int, float]], optional): The number
             of seconds to wait for the connection. Defaults to None.
+        middleware (Optional[List[HttpClientMiddlewareCallback]], optional):
+            Optional middleware. Defaults to None.
 
     Raises:
         HTTPError: Is the status code is not ok.
@@ -617,28 +505,30 @@ async def post_json(
 
     headers = [] if headers is None else list(headers)
 
-    if not header.find(b'accept', headers):
-        headers.append((b'accept', b'application/json'))
+    if not header.find(header.ACCEPT, headers):
+        headers.append((header.ACCEPT, b'application/json'))
 
     content = dumps(obj)
-    if not header.find(b'content-type', headers):
-        headers.append((b'content-type', b'application/json'))
+    if not header.find(header.CONTENT_TYPE, headers):
+        headers.append((header.CONTENT_TYPE, b'application/json'))
 
-    text = await post_text(
-        url,
-        text=content,
-        encoding='utf-8',
-        headers=headers,
-        loop=loop,
-        cafile=cafile,
-        capath=capath,
-        cadata=cadata,
-        ssl_context=ssl_context,
-        decompressors=decompressors,
-        protocols=protocols,
-        ciphers=ciphers,
-        options=options,
-        chunk_size=chunk_size,
-        connect_timeout=connect_timeout
-    )
-    return loads(text)
+    data = text_writer(content, chunk_size=chunk_size) if content else None
+
+    async with HttpClient(
+            url,
+            method='POST',
+            headers=headers,
+            body=data,
+            loop=loop,
+            cafile=cafile,
+            capath=capath,
+            cadata=cadata,
+            ssl_context=ssl_context,
+            protocols=protocols,
+            ciphers=ciphers,
+            options=options,
+            connect_timeout=connect_timeout,
+            middleware=middleware
+    ) as response:
+        await response.raise_for_status()
+        return await response.json(loads)
